# Generated by Django 2.2.13 on 2020-12-30 18:47

from django.db import migrations, models


class Migration(migrations.Migration):

    initial = True

    dependencies = [
    ]

    operations = [
        migrations.CreateModel(
            name='FileMetadata',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('inode', models.IntegerField(db_index=True, verbose_name='inode')),
                ('filename', models.TextField(blank=True, null=True, verbose_name='Filename')),
                ('suffix', models.CharField(blank=True, max_length=10, null=True, verbose_name='Suffix')),
                ('path', models.TextField(blank=True, null=True, verbose_name='Path')),
                ('size', models.IntegerField(blank=True, null=True, verbose_name='Size (bits)')),
                ('size_str', models.CharField(blank=True, max_length=10, null=True, verbose_name='Size')),
                ('created', models.DateTimeField(auto_now_add=True, verbose_name='Created')),
                ('modified', models.DateTimeField(auto_now_add=True, verbose_name='Modified')),
                ('title', models.CharField(blank=True, max_length=256, null=True, verbose_name='Title')),
                ('description', models.CharField(blank=True, max_length=256, null=True, verbose_name='Description')),
                ('text_content', models.TextField(blank=True, null=True, verbose_name='Text content')),
            ],
            options={
                'verbose_name': 'static file',
                'verbose_name_plural': 'static files',
            },
        ),
    ]
